"
COPYRIGHT (c) 2021 LabWare

jv:libgdbs - GNU Debugger Interface Library

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
"
"{ Package: 'jv:libgdbs/tests' }"

"{ NameSpace: Smalltalk }"

GDBDebuggerTestCase subclass:#GDBDebuggerExamples
	instanceVariableNames:'gdb'
	classVariableNames:''
	poolDictionaries:''
	category:'GDB-Core-Examples'
!

!GDBDebuggerExamples class methodsFor:'documentation'!

copyright
"
COPYRIGHT (c) 2021 LabWare

jv:libgdbs - GNU Debugger Interface Library

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
"
!

documentation
"
    Commented examples of jv:libgdbs API.

    [author:]
        Jan Vrany <jan.vrany@labware.com>

    [instance variables:]

    [class variables:]

    [see also:]

"
! !

!GDBDebuggerExamples class methodsFor:'accessing'!

isTestSelector:aSelector

    ^aSelector notNil and:[aSelector startsWith:'example']
! !

!GDBDebuggerExamples methodsFor:'examples'!

example01_inject_code_riscv
    "
    This example demonstrates how to use libgdbs to dynamically
    inject code into a debugee. This example uses RISC-V as a target 
    architecture.

    At any point, you may open a VDB on debugger instance by evaluating

        VDBDebuggerApplication openFor: gdb
    "

    | fatshell nzone code exit |

    "First, we need a 'fatshell'. On Debian, you may compile it by

         cd .../jv/libgdbs/tests/c
         make make CC=riscv64-linux-gnu-gcc BUILD_TARGET=riscv64-unknown-linux-gnu

    To see fatshell source, inspect:

        ((Smalltalk getPackageDirectoryForPackage:'jv:libgdbs/tests') / 'c' / 'fatshell.c') asFilename contents asString

    "
    fatshell := (Smalltalk getPackageDirectoryForPackage:'jv:libgdbs/tests') / 'c' / 'riscv64-unknown-linux-gnu' / 'fatshell'.

    self skipIf: fatshell exists not 
         description: 'fatshell for RISC-V does not exists'.
    self skipIf: (OperatingSystem pathOfCommand: 'qemu-riscv64-static') isNil 
         description: 'QEMU not found'.

    "First, start QEMU: "
    TerminalView 
        openOnCommand:'qemu-riscv64-static -g 1234 ', fatshell pathName, ' 1 2 3 4'
        onExit:[:vt | vt topView close ]. 

    "...then create a new GDB and connect to QEMU: "
    gdb := GDBDebugger new.
    gdb executable: fatshell.
    gdb targetConnect: 'remote' parameters: #('localhost:1234').

    "Now let's determine where the nzone is: "
    nzone := ((gdb evaluate: '(void*)&nzone') valueFormatted: GDBOutputFormat signedDecimal) asInteger.

    "Now, let's generate some code, Following might be generated on
     the fly by smalltalk code (or anything else), but here we will
     just use a simple static code:"
    code := ByteArray fromHexStringWithSeparators: 
            '93 07 05 00',    "/ mv      a5,a0
            '13 05 10 00',    "/ li      a0,1
            '93 06 10 00',    "/ li      a3,1
            '63 8a a7 00',    "/ beq     a5,a0,+20 ---------+
            '13 87 07 00',    "/ mv      a4,a5     <---+    |
            '9b 87 f7 ff',    "/ addiw   a5,a5,-1      |    |
            '3b 05 a7 02',    "/ mulw    a0,a4,a0      |    |
            'e3 9a d7 fe',    "/ bne     a5,a3,-12 ----+    |
            '67 80 00 00'.    "/ ret               <--------+

    "Now inject the code into debuggee by owerwriting nzone: "
    gdb selectedInferior memoryAt: nzone put: code.

    "Finally, run the inferior"
    exit := gdb send: 'c' andWaitFor: GDBThreadGroupExitedEvent.

    "This is here so you can place a breakpoint on this line :-)"
    exit yourself.

    "Created: / 24-03-2021 / 12:40:17 / Jan Vrany <jan.vrany@labware.com>"
    "Modified: / 27-03-2021 / 07:01:57 / Jan Vrany <jan.vrany@labware.com>"
!

example_README
    "
    This is an example from libgdb's README.md.

    If updated, please update the README.md as well.
    "

    | b0 thread frame i r |

    "Create new debugger object - `gdb`."
    gdb := GDBDebugger new.

    "Load an executable to debug, the parameter is a String
     with path to the executable."
    gdb executable: GDBDebuggeesResource current binaryFactorial1.

    "Set breakpoint on function `factorial(int)`, see`factorial.c`(*). 
     Here we use `GDBDebugger >> #send:` to send a standard CLI command
     to `gdb`. The method blocks and returns once the command is executed.
     See class `GDBDebugger`, protocol 'commands' for more ways to send a
     command to GDB.

     (*) https://swing.fit.cvut.cz/hg/jv-libgdbs/file/tip/tests/c/factorial1.c
    "
    gdb send: 'b factorial'.

    "You may ask `gdb` for list of breakpoints, ..."
    gdb breakpoints.

    "...retrieve a breakpoint as an object..."
    b0 := gdb breakpoints first.

    "...and then use breakpoint object to get details
     for example on what file and line the breakpoint was
     put:"
    'location is %1 : %2' bindWith: b0 file with: b0 line.

    "Breakpoint objects can also be used to manipulate it,
     for example to set a condition when the breakpoint should
     hit. See class `GDBBreakpoint`. "
    b0 condition: 'i == 1'.

    "Now let's run debuggee util it hits the breakpoint using
     GDB's `r` (or `run`) command. Here we (have to) use a different 
     API than plain `#send: - `#send:` executes the command and return 
     as soon as the command is executed. This is not what we want now,
     here we want to run the debuggee and wait until breakpoint hits
     (or until some error occur). In theory, this could be hours after
     the `r` command is executed.

     For cases like this, libgdb's provides another API:
     `GDBDebugger >> send:andWaitFor:` to wait for 'stopped' event:"
    gdb send: 'r' andWaitFor: GDBStoppedEvent.

    "Now let's see where we have stopped. Since GDB allows
     to debug multiple processes (called inferiors in GDB parlance)
     and each process may have multiple threads, we have to first get
     a the thread and then ask the thread for first (newest) frame of
     threads callstack. In this case, there' only one process with
     only one thread, so getting the thread is easy.

     Note, that there's always a callstack with at least one frame."
    thread := gdb selectedInferior threads first.
    frame := thread stack first.

    "Again, thread and frame are objects and can be used to access 
     program state"
    frame func.    " -> 'factorial'"
    i := frame variables first.
    i name.        " -> 'i' "
    i value.       " -> '1' "

    r := frame registers first.
    r name.        " -> 'rax' (depending on target architecture, of course) "
    r value.

    "Finally, remove all breakpoints and let the program continue and finish..."
    gdb send: 'del'.     
    gdb send: 'c' andWaitFor: GDBThreadGroupExitedEvent.

    "Modified (comment): / 22-02-2021 / 20:10:33 / Jan Vrany <jan.vrany@labware.com>"
! !

!GDBDebuggerExamples methodsFor:'running'!

setUp


    "Created: / 18-10-2018 / 10:34:09 / Jan Vrany <jan.vrany@fit.cvut.cz>"
    "Modified (format): / 19-10-2018 / 09:50:45 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

tearDown
    (gdb notNil and:[ gdb isConnected ]) ifTrue:[ 
        gdb send: 'quit' andWait: false.
    ].
    super tearDown.

    "Created: / 19-01-2018 / 09:18:42 / jv"
    "Modified: / 18-10-2018 / 10:34:28 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!GDBDebuggerExamples class methodsFor:'documentation'!

version_HG

    ^ '$Changeset: <not expanded> $'
! !

